@@ -3,7 +3,9 @@
 import basic_qp.generate_problem
 import basic_qp2.generate_problem
 import lin_alg.generate_problem
-import solve_linsys.generate_problem
-import primal_infeasibility.generate_problem
+import non_cvx.generate_problem
 import primal_dual_infeasibility.generate_problem
+import primal_infeasibility.generate_problem
+import solve_linsys.generate_problem
+import unconstrained.generate_problem
 import update_matrices.generate_problem
